from .enums.StiDatabaseType import StiDatabaseType
from .enums.StiDataType import StiDataType
from .StiFileAdapter import StiFileAdapter


class StiCsvAdapter(StiFileAdapter):
    
### Properties

    version = '2026.1.7'
    """Current version of the data adapter."""

    checkVersion = True
    """Sets the version matching check on the server and client sides."""

    type = StiDatabaseType.CSV
    dataType = StiDataType.CSV



from .classes.StiDataResult import StiDataResult
from .enums import StiDatabaseType
from .events.StiConnectionEventArgs import StiConnectionEventArgs
from .StiSqlAdapter import StiSqlAdapter


class StiMySqlAdapter(StiSqlAdapter):

### Properties

    version = '2026.1.7'
    """Current version of the data adapter."""

    checkVersion = True
    """Sets the version matching check on the server and client sides."""
    
    type = StiDatabaseType.MYSQL
    driverName = 'mysql-connector-python'
    

### Methods

    def connect(self):
        if self.connectionInfo.driver:
            return self.connectOdbc()
        
        if not self.connectionInfo.charset:
            self.connectionInfo.charset = 'utf8'
        
        try:
            args = StiConnectionEventArgs(self.handler.request, self.type, self.driverName, self.connectionInfo)
            self.handler.onDatabaseConnect(args)

            if args.link != None:
                self.connectionLink = args.link
            else:
                from mysql.connector.connection import MySQLConnection
                self.connectionLink = MySQLConnection(
                    user = self.connectionInfo.userId,
                    password = self.connectionInfo.password,
                    host = self.connectionInfo.host,
                    database = self.connectionInfo.database,
                    port = self.connectionInfo.port,
                    charset = self.connectionInfo.charset)
        except Exception as e:
            return StiDataResult.getError(str(e)).getDataAdapterResult(self)
        
        return StiDataResult.getSuccess().getDataAdapterResult(self)
    
    def process(self):
        if super().process():
            return True

        self.connectionInfo.port = 3306

        parameterNames = {
            'driver': ['driver'],
            'host': ['server', 'host', 'location'],
            'port': ['port'],
            'database': ['database', 'data source', 'dbname'],
            'userId': ['uid', 'user', 'username', 'userid', 'user id'],
            'password': ['pwd', 'password'],
            'charset': ['charset']
        }

        return self.processParameters(parameterNames)

    def getType(self, meta: tuple):
        if self.connectionInfo.driver:
            return super().getType(meta)
        
        from mysql.connector import FieldFlag, FieldType

        types = {
            'tiny': [FieldType.TINY],
            'int': [FieldType.BIT, FieldType.SHORT, FieldType.LONG, FieldType.LONGLONG, FieldType.INT24, FieldType.YEAR],
            'number': [FieldType.DECIMAL, FieldType.FLOAT, FieldType.DOUBLE, FieldType.NEWDECIMAL],
            'time': [FieldType.TIME],
            'datetime': [FieldType.TIMESTAMP, FieldType.DATE, FieldType.DATETIME, FieldType.NEWDATE],
            'array': [FieldType.TINY_BLOB, FieldType.MEDIUM_BLOB, FieldType.LONG_BLOB, FieldType.GEOMETRY, FieldType.NULL],
            'blob': [FieldType.BLOB],
            'string': [FieldType.STRING, FieldType.VARCHAR, FieldType.VAR_STRING, FieldType.SET, FieldType.ENUM, FieldType.JSON]
        }

        for key, array in types.items():
            if meta[1] in array:
                if key == 'tiny':
                    return 'int'  # boolean?
                if key == 'blob':
                    return 'array' if meta[7] & FieldFlag.BINARY else 'string'
                return key

        return 'string'
    
    def makeQuery(self, procedure: str, parameters: list):
        paramsString = super().makeQuery(procedure, parameters)
        return f'CALL {procedure} ({paramsString})'
    


import codecs
import urllib.parse

from ..classes.StiDataResult import StiDataResult
from ..classes.StiFunctions import StiFunctions
from ..classes.StiParameter import StiParameter
from ..enums.StiBaseEventType import StiBaseEventType
from ..enums.StiDatabaseType import StiDatabaseType
from ..enums.StiDataCommand import StiDataCommand
from ..enums.StiFrameworkType import StiFrameworkType
from ..events.StiDataEventArgs import StiDataEventArgs
from ..events.StiEvent import StiEvent
from ..StiDataAdapter import StiDataAdapter
from .StiBaseRequest import StiBaseRequest
from .StiBaseResponse import StiBaseResponse
from .StiBaseResult import StiBaseResult


class StiBaseHandler:
    """
    Event handler for all requests from the report generator. Processes the incoming request, communicates with data adapters, 
    prepares parameters and triggers events, and performs all necessary actions. After this, the event handler prepares a 
    response for the web server.
    """

### Fields

    __events: dict = None
    __url: str = None


### Properties

    version = '2026.1.7'
    """Current version of the event handler."""
    
    checkDataAdaptersVersion = True
    """Enables checking for client-side and server-side data adapter versions to match."""

    framework = StiFrameworkType.DEFAULT
    origin: str = None
    query: dict = None
    body: str = None
    cookies: dict = None
    error: str = None
    request: StiBaseRequest = None
    frameworkRequest: object = None
    dataAdapter: StiDataAdapter = None

    passQueryParameters = False
    """Enables automatic passing of GET parameters from the current URL to event handler requests."""

    encryptSqlData = True
    """
    Enables encryption of SQL data transmitted from the server to the report generator.
    This improves security but slows down the processing of large data.
    """
    
    @property
    def url(self):
        #return self.__url or ''
        
        url = self.__url or ''
        if self.passQueryParameters:
            pass
            """
            foreach ($_GET as $key => $value)
                if (strpos($url, $key) === false) {
                    $url .= strpos($url, '?') === false ? '?' : '&';
                    $url .= "$key=" . rawurlencode($value);
                }
            """

        return url
    
    @url.setter
    def url(self, value):
        self.__url = value


### Events

    @property
    def onDatabaseConnect(self) -> StiEvent:
        """The event is invoked before connecting to the database after all parameters have been received. Only Python functions are supported."""
        return self._getEvent('onDatabaseConnect')

    @onDatabaseConnect.setter
    def onDatabaseConnect(self, value):
        self._setEvent('onDatabaseConnect', value)


    @property
    def onBeginProcessData(self) -> StiEvent:
        """The event is invoked before data request, which needed to render a report. Python and JavaScript functions are supported."""
        return self._getEvent('onBeginProcessData')

    @onBeginProcessData.setter
    def onBeginProcessData(self, value):
        self._setEvent('onBeginProcessData', value)


    @property
    def onEndProcessData(self) -> StiEvent:
        """The event is invoked after loading data before rendering a report. Python and JavaScript functions are supported."""
        return self._getEvent('onEndProcessData')

    @onEndProcessData.setter
    def onEndProcessData(self, value):
        self._setEvent('onEndProcessData', value)


### Helpers

    def __setQuery(self, query: dict | str) -> None:
        if query != None:
            self.query = query if isinstance(query, dict) else dict(urllib.parse.parse_qsl(query))

    def __setBody(self, body: bytes | str) -> None:
        if body != None:
            self.body = body if type(body) == str else codecs.decode(body)

    def __removeResourceParameters(self, url: str) -> str:
        query = urllib.parse.urlparse(url).query
        if StiFunctions.isNullOrEmpty(query):
            return url
        
        params = []
        remove = ['sti_event', 'sti_data']
        for param, values in urllib.parse.parse_qs(query).items():
            if param not in remove:
                for value in values:
                    params.append((param, value))

        baseUrl = url[:url.find('?')]
        return baseUrl if len(params) == 0 else baseUrl + '?' + urllib.parse.urlencode(params)
    
    def _createRequest(self):
        return StiBaseRequest()
    
    def _checkEvent(self):
        return self.request.event.value in StiBaseEventType.getValues(True)
    
    def _checkCommand(self):
        return self.request.command.value in StiDataCommand.getValues()

    def _getEvent(self, name) -> StiEvent:
        event = self.__events.get(name)
        if event == None:
            event = StiEvent(self, name)
            self.__events[name] = event

        return event
    
    def _setEvent(self, name, value):
        if isinstance(value, StiEvent):
            self.__events[name] = value
        elif callable(value) or isinstance(value, (bool, str)):
            self._getEvent(name).append(value)

    def _setEvents(self, events: dict):
        self.__events = events


### Process

    def __processFrameworkRequest(self, request: object) -> bool:
        try:
            from django.http import HttpRequest as DjangoRequest
            if isinstance(request, DjangoRequest):
                # This code is needed to update CSRF token in cookies
                from django.middleware.csrf import get_token
                get_token(request)

                self.__setQuery(request.GET)
                self.__setBody(request.body)
                self.url = self.url or self.__removeResourceParameters(request.build_absolute_uri())
                self.origin = '{0}://{1}'.format(request.scheme, request.get_host())
                self.cookies = request.COOKIES
                self.framework = StiFrameworkType.DJANGO
                self.frameworkRequest = request
                return True
        except Exception as e:
            if not isinstance(e, ModuleNotFoundError):
                self.error = 'Request: ' + str(e)
                return False

        try:
            from flask import Request as FlaskRequest
            if isinstance(request, FlaskRequest):
                self.__setQuery(request.args.to_dict())
                self.__setBody(request.get_data(False))
                self.url = self.url or self.__removeResourceParameters(request.base_url)
                self.origin = request.origin
                self.cookies = request.cookies
                self.framework = StiFrameworkType.FLASK
                self.frameworkRequest = request
                return True
        except Exception as e:
            if not isinstance(e, ModuleNotFoundError):
                self.error = 'Request: ' + str(e)
                return False

        try:
            from tornado.httputil import HTTPServerRequest as TornadoRequest
            if isinstance(request, TornadoRequest):
                self.__setQuery(request.query)
                self.__setBody(request.body)
                self.url = self.url or self.__removeResourceParameters(request.full_url())
                self.origin = '{0}://{1}'.format(request.protocol, request.host)
                self.cookies = request.cookies
                self.framework = StiFrameworkType.TORNADO
                self.frameworkRequest = request
                return True
        except Exception as e:
            if not isinstance(e, ModuleNotFoundError):
                self.error = 'Request: ' + str(e)
                return False
        
        self.error = 'Unsupported request: ' + str(request.__class__)
        return False
    
    def __processParameters(self):
        parameters = dict()
        if len(self.request.parameters or '') > 0 and len(self.request.queryString or '') > 0:
            for item in self.request.parameters:
                name = item['name'][1:] if item['name'].startswith(('@', ':')) else item['name']
                parameters[name] = StiParameter(name, item.get('type', 0), item.get('typeName'), item.get('typeGroup'), item.get('size', 0), item.get('value'))

        self.request.parameters = parameters

    def processRequest(self, request: object = None, query: dict | str = None, body: bytes | str = None) -> bool:
        """
        Processing an HTTP request from the client side of the component. If successful, it is necessary to return a response 
        with the processing result, which can be obtained using the 'getResponse()' or 'getFrameworkResponse()' functions.
        
        request:
            A request object for one of the supported frameworks.

        query:
            The GET query string if no framework request is specified.
        
        body:
            The POST form data if no framework request is specified.

        return:
            True if the request was processed successfully.
        """

        self.error = None

        if request != None:
            self.__processFrameworkRequest(request)
            if (self.error != None):
                return False
        
        if self.framework == StiFrameworkType.DEFAULT:
            try:
                self.__setQuery(query)
                self.__setBody(body)
            except Exception as e:
                self.error = 'Request: ' + str(e)
                return False
        
        self.request = self._createRequest()
        self.request.process(self.query, self.body)
        if len(self.request.error or '') > 0:
            self.error = self.request.error
            return False
        
        if not self._checkEvent():
            self.error = f'Unknown event: {self.request.event.value}'
            return False
        
        if not self._checkCommand():
            self.error = f'Unknown command: {self.request.command.value}'
            return False
        
        return True
    

### Results

    def __getSupportedDataAdaptersResult(self) -> StiBaseResult:
        result: StiBaseResult = StiBaseResult.getSuccess()
        result.types = StiDatabaseType.getValues()
        result.handlerVersion = self.version
        return result
    
    def __getDataResult(self, result: StiBaseResult, notice: str, args: StiDataEventArgs) -> StiBaseResult:

        # The event did not return any result, the result of the data adapter is used
        if result == None:
            result = args.result

        # Copying message from event if data adapter message is empty
        if StiFunctions.isNullOrEmpty(args.result.notice):
            args.result.notice = result.notice if not StiFunctions.isNullOrEmpty(result.notice) else notice

        # If the result from the event is successful, use the result of the data adapter
        if result.success:
            return args.result

        # Passing data adapter parameters
        if isinstance(result, StiDataResult):
            result = result.getDataAdapterResult(self.dataAdapter)

        return result
    
    def __getDataAdapterResult(self):
        args = StiDataEventArgs(self.request)
        result = self.onBeginProcessData.getResult(args)
        if result != None and not result.success:
            return result
        
        # Saving a message to return in the results of the next events
        notice = result.notice if result != None else None

        # Prepare the connection string or the URL of the data file
        connectionString = args.connectionString
        if self.request.command == StiDataCommand.GET_DATA or self.request.command == StiDataCommand.GET_SCHEMA:
            connectionString = args.pathData if self.request.command == StiDataCommand.GET_DATA else args.pathSchema

        # Get the necessary data adapter
        self.dataAdapter: StiDataAdapter = StiDataAdapter.getDataAdapter(args.database, connectionString)
        if self.dataAdapter == None:
            return StiBaseResult.getError(f'Unknown database type: {args.database}')
        
        self.dataAdapter.handler = self

        # Get the data source schema
        if self.request.command == StiDataCommand.RETRIEVE_SCHEMA:
            args.result = self.dataAdapter.getDataResult(args.dataSource, args.maxDataRows)
            result = self.onEndProcessData.getResult(args, StiDataResult)
            return self.__getDataResult(result, notice, args)

        # Process SQL data
        if self.request.command == StiDataCommand.EXECUTE or self.request.command == StiDataCommand.EXECUTE_QUERY:

            # The MongoDB data source does not contain a connection string
            # Using the data source name to find a match from the MongoDB data object
            try:
                from ..StiMongoDbAdapter import StiMongoDbAdapter
                if isinstance(self.dataAdapter, StiMongoDbAdapter):
                    args.queryString = args.dataSource
            except:
                pass

            if self.request.command == StiDataCommand.EXECUTE:
                args.queryString = self.dataAdapter.makeQuery(args.queryString, args.parameters)

            if len(args.parameters) > 0:
                from ..StiSqlAdapter import StiSqlAdapter
                args.queryString = StiSqlAdapter.applyQueryParameters(args.queryString, args.parameters, self.request.escapeQueryParameters)

            args.result = self.dataAdapter.getDataResult(args.queryString, args.maxDataRows)
            result = self.onEndProcessData.getResult(args, StiDataResult)
            return self.__getDataResult(result, notice, args)

        # Process file data
        if self.request.command == StiDataCommand.GET_DATA or self.request.command == StiDataCommand.GET_SCHEMA:
            args.result = self.dataAdapter.getDataResult(connectionString)
            result = self.onEndProcessData.getResult(args, StiDataResult)

            # If the server side event is not set, the result is always successful
            # Required for loading file data on the JavaScript client-side
            if not self.onEndProcessData.hasServerCallbacks():
                args.result.success = True

            return self.__getDataResult(result, notice, args)
        
        return self.dataAdapter.test()

    def getFrameworkResponse(self, handler = None) -> object:
        """Returns the result of processing a request from the client side intended for one of the supported frameworks."""

        return self.getResponse().getFrameworkResponse(handler)

    def getResponse(self) -> StiBaseResponse:
        """
        Returns the result of processing a request from the client side. The response object will contain the data for the response, 
        as well as their MIME type, Content-Type, and other useful information to create a web server response.
        """
        
        return StiBaseResponse(self)

    def getResult(self):
        """
        Returns the result of processing a request from the client side. The result object will contain a collection of data, 
        message about the result of the command execution, and other technical information.
        """

        if len(self.error or '') > 0:
            result: StiBaseResult = StiBaseResult.getError(self.error)
            result.handlerVersion = self.version
            return result
        
        if self.request.command == StiDataCommand.GET_SUPPORTED_ADAPTERS:
            return self.__getSupportedDataAdaptersResult()

        self.__processParameters()
        result = self.__getDataAdapterResult()
        result.handlerVersion = self.version
        return result
    

### Constructor

    def __init__(self, url: str = None):
        self.__events = dict()
        self.__url = url



import os

import requests


class StiPath:

### Fields

    filePath: str = None
    """The full path to the file, if the file exists."""

    directoryPath: str = None
    """The full path to the file directory, if the file exists."""

    fileName: str = None
    """The file name."""

    fileNameOnly: str = None
    """The file name without the extension."""

    fileExtension: str = None
    """The file extension."""

    fileUrl: str = None
    """The URL to the file, if the file exists (code 200 was received when requesting the file)."""


### Helpers

    def normalize(path: str) -> str:
        return os.path.normpath((path or '').split('?')[0]).rstrip('/\\')
    
    def __isUrl(path: str) -> bool:
        return path != None and (path.startswith('http://') or path.startswith('https://'))
    
    def __getMissingFileName(filePath: str) -> str:
        filePath = StiPath.normalize(filePath)
        return os.path.basename(filePath)
    
    def __getRealFilePath(filePath: str) -> str:
        if StiPath.__isUrl(filePath):
            try:
                headers = requests.head(filePath)
            except Exception as e:
                return None
            
            return filePath if headers.status_code == 200 else None

        filePath = StiPath.normalize(filePath)
        if os.path.isfile(filePath):
            return filePath
        
        workingDir = os.getcwd()
        filePath = StiPath.normalize(f'{workingDir}/{filePath}')
        if os.path.isfile(filePath):
            return filePath
        
        return None
    
    def __getRealDirectoryPath(directoryPath: str) -> str:
        if StiPath.__isUrl(directoryPath):
            return None

        filePath = StiPath.normalize(directoryPath)
        
        directoryPath = filePath
        if os.path.isdir(directoryPath):
            return directoryPath

        workingDir = os.getcwd()
        directoryPath = StiPath.normalize(f'{workingDir}/{directoryPath}')
        if os.path.isdir(directoryPath):
            return directoryPath
        
        directoryPath = os.path.dirname(filePath)
        if os.path.isdir(directoryPath):
            return directoryPath

        directoryPath = StiPath.normalize(f'{workingDir}/{directoryPath}')
        if os.path.isdir(directoryPath):
            return directoryPath
        
        return None


### Constructor

    def __init__(self, filePath):
        self.filePath = StiPath.__getRealFilePath(filePath)
        self.directoryPath = StiPath.__getRealDirectoryPath(filePath)
        if self.filePath != None and StiPath.__isUrl(filePath):
            self.fileUrl = self.filePath
        
        self.fileName = os.path.basename(self.filePath) if self.filePath != None else StiPath.__getMissingFileName(filePath)
        if self.filePath == None and (self.directoryPath or '').endswith(self.fileName):
            self.fileName = None

        if self.fileName != None:
            self.fileNameOnly, self.fileExtension = os.path.splitext(self.fileName)
            self.fileExtension = self.fileExtension[1:].lower() if len(self.fileExtension or '') > 1 else ''


from __future__ import annotations

import codecs
import json
import typing

from ..classes.StiDataResult import StiDataResult
from ..enums.StiDataType import StiDataType
from ..enums.StiFrameworkType import StiFrameworkType
from .StiBaseResult import StiBaseResult

if typing.TYPE_CHECKING:
    from .StiBaseHandler import StiBaseHandler


class StiBaseResponse:
    """
    The result of executing an event handler request. 
    You can get the data, its type and other parameters necessary to create a web server response.
    """

### Properties

    handler: StiBaseHandler = None
    result: StiBaseResult = None

    @property
    def origin(self) -> str:
        """Returns the detected origin url for the handler response. Can be used for the 'Access-Control-Allow-Origin' header of the response."""

        return self.handler.origin

    @property
    def mimeType(self) -> str:
        """Returns the MIME type for the handler response."""

        if isinstance(self.result, StiDataResult) and self.result.dataType != None:
            return self.result.dataType

        return StiDataType.JSON.value
    
    @property
    def contentType(self) -> str:
        """Returns the content type for the handler response. Can be used for the 'Content-Type' header of the response."""

        return self.mimeType + '; charset=utf-8'
    
    @property
    def data(self) -> bytes:
        """Returns the handler response as a byte array. When using encryption, the response will be encrypted and encoded into a Base64 string."""

        if isinstance(self.result, StiDataResult) and self.result.type == 'File':
            return (self.result.data or '').encode()

        data = json.dumps(self.result, default = StiBaseResult.getProperties)
        encryptSqlData = self.handler.encryptSqlData or self.result.type != 'SQL'
        if self.handler.request.encryptData and encryptSqlData:
            data = codecs.encode(codecs.encode(data.encode(), 'base64').decode(), 'rot13')
            
        return data.encode()


### Response

    def getFrameworkResponse(self, handler = None):
        """Returns a response intended for one of the supported frameworks."""
        
        if self.handler.framework == StiFrameworkType.DJANGO:
            try:
                from django.http import HttpResponse
                contentType = self.contentType
                response = HttpResponse(self.data, content_type = contentType)
                response.headers['Access-Control-Allow-Origin'] = self.origin
                response.headers['X-Stimulsoft-Result'] = self.result.type
                return response
            except:
                return self

        if self.handler.framework == StiFrameworkType.FLASK:
            try:
                from flask import make_response
                response = make_response(self.data)
                response.headers.set('Content-Type', self.contentType)
                response.headers.set('Access-Control-Allow-Origin', self.origin)
                response.headers.set('X-Stimulsoft-Result', self.result.type)
                return response
            except:
                return self
            
        if self.handler.framework == StiFrameworkType.TORNADO:
            try:
                from tornado.web import RequestHandler as TornadoRequestHandler
                if isinstance(handler, TornadoRequestHandler):
                    handler.set_header('Content-Type', self.contentType)
                    handler.set_header('Access-Control-Allow-Origin', self.origin)
                    handler.set_header('X-Stimulsoft-Result', self.result.type)
                    handler.write(self.data)
                    return None

                return self
            except:
                return self

        return self
    

### Constructor

    def __init__(self, handler: StiBaseHandler, result: StiBaseResult = None):
        self.handler = handler
        self.result = result if result != None else self.handler.getResult()



from __future__ import annotations

import typing

from pyodbc import Connection

from .classes.StiDataResult import StiDataResult
from .classes.StiFunctions import StiFunctions
from .enums.StiDatabaseType import StiDatabaseType

if typing.TYPE_CHECKING:
    from .classes.StiBaseHandler import StiBaseHandler


class StiDataAdapter:

### Properties

    version = '2026.1.7'
    """Current version of the data adapter."""

    checkVersion = False
    """Sets the version matching check on the server and client sides."""

    handler: StiBaseHandler = None
    
    type = StiDatabaseType.NONE
    """The type of database processed by the data adapter."""

    driverName: str = None
    """The name of the current Python data driver."""

    connectionString: str = None
    """The connection string or URL for the current data source."""

    connectionLink: Connection = None
    """Link to the database connection driver."""


### Methods

    def connect(self):
        return StiDataResult.getSuccess().getDataAdapterResult(self)
    
    def disconnect(self):
        if self.connectionLink != None:
            self.connectionLink.close()
        self.connectionLink = None

    def test(self):
        result = self.connect()
        if result.success:
            self.disconnect()
        return result
    
    def process(self):
        return False

    def getDataResult(self, queryString: str, maxDataRows: int = -1) -> StiDataResult:
        return StiDataResult.getSuccess().getDataAdapterResult(self)
    

### Helpers

    @staticmethod
    def getDataAdapter(database: StiDatabaseType, connectionString: str):
        if database == StiDatabaseType.MYSQL:
            from .StiMySqlAdapter import StiMySqlAdapter
            return StiMySqlAdapter(connectionString)
        
        elif database == StiDatabaseType.MSSQL:
            from .StiMsSqlAdapter import StiMsSqlAdapter
            return StiMsSqlAdapter(connectionString)
        
        elif database == StiDatabaseType.FIREBIRD:
            from .StiFirebirdAdapter import StiFirebirdAdapter
            return StiFirebirdAdapter(connectionString)
        
        elif database == StiDatabaseType.POSTGRESQL:
            from .StiPostgreSqlAdapter import StiPostgreSqlAdapter
            return StiPostgreSqlAdapter(connectionString)
        
        elif database == StiDatabaseType.ORACLE:
            from .StiOracleAdapter import StiOracleAdapter
            return StiOracleAdapter(connectionString)
        
        elif database == StiDatabaseType.ODBC:
            from .StiOdbcAdapter import StiOdbcAdapter
            return StiOdbcAdapter(connectionString)
        
        elif database == StiDatabaseType.MONGODB:
            from .StiMongoDbAdapter import StiMongoDbAdapter
            return StiMongoDbAdapter(connectionString)
        
        elif database == StiDatabaseType.XML:
            from .StiXmlAdapter import StiXmlAdapter
            return StiXmlAdapter(connectionString)
        
        elif database == StiDatabaseType.JSON:
            from .StiJsonAdapter import StiJsonAdapter
            return StiJsonAdapter(connectionString)
        
        elif database == StiDatabaseType.CSV:
            from .StiCsvAdapter import StiCsvAdapter
            return StiCsvAdapter(connectionString)
        
        return None
    

### Constructor
    
    def __init__(self, connectionString: str):
        self.connectionString = connectionString.strip() if not StiFunctions.isNullOrEmpty(connectionString) else None
        self.process()


import pyodbc
from pyodbc import Connection

from .classes.StiDataResult import StiDataResult
from .enums import StiDatabaseType
from .events.StiConnectionEventArgs import StiConnectionEventArgs
from .StiSqlAdapter import StiSqlAdapter


class StiOdbcAdapter(StiSqlAdapter):

### Properties

    version = '2026.1.7'
    """Current version of the data adapter."""

    checkVersion = True
    """Sets the version matching check on the server and client sides."""
    
    connectionLink: Connection
    type = StiDatabaseType.ODBC
    driverName = 'pyodbc'


### Methods

    def connect(self):
        try:
            args = StiConnectionEventArgs(self.handler.request, self.type, self.driverName, self.connectionInfo)
            self.handler.onDatabaseConnect(args)

            if args.link != None:
                self.connectionLink = args.link
            else:    
                self.connectionLink = pyodbc.connect(self.connectionString)
                if self.connectionInfo.charset:
                    self.connectionLink.setdecoding(pyodbc.SQL_CHAR, self.connectionInfo.charset)
                    self.connectionLink.setdecoding(pyodbc.SQL_WCHAR, self.connectionInfo.charset)
        except Exception as e:
            return StiDataResult.getError(str(e)).getDataAdapterResult(self)
        
        return StiDataResult.getSuccess().getDataAdapterResult(self)
    
    def process(self):
        if super().process():
            return True

        parameterNames = {
            'driver': ['driver'],
            'userId': ['uid', 'user', 'username', 'userid', 'user id'],
            'password': ['pwd', 'password']
        }

        return self.processParameters(parameterNames)
    
    


from setuptools import setup, find_packages

with open('README.md') as file:
    readmeFile = file.read()

with open('REQUIREMENTS.txt') as file:
    requiresFile = [text.strip() for text in file if text.strip()]

setup(
    name = 'stimulsoft_data_adapters',
    version = '2026.1.7',
    author = 'Stimulsoft',
    author_email = 'info@stimulsoft.com',
    description = 'Stimulsoft data adapters for Python.',
    long_description = readmeFile,
    long_description_content_type = 'text/markdown',
    url = 'https://www.stimulsoft.com/en/products/reports-python',
    license = 'https://www.stimulsoft.com/en/licensing/developers',
    classifiers=[
        'License :: Other/Proprietary License',
        'Framework :: Django',
        'Framework :: Flask',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Database',
        'Topic :: Software Development'
    ],
    install_requires = ['pyodbc', 'requests'],
    extras_require = {'ext': requiresFile},
    packages = find_packages(),
    python_requires = '>=3.10'
)



import json
import re
import uuid
from enum import Enum


class StiFunctions:

### String

    def getJavaScriptValue(value) -> str:
        if value is None:
            return 'null'
        
        if isinstance(value, Enum):
            return 'null' if value.value is None else value.value
        
        return json.dumps(value, ensure_ascii=False).replace('\\/', '/')

    def isNullOrEmpty(value) -> bool:
        return len(value or '') == 0
    
    def newGuid(length = 16) -> str:
        return uuid.uuid4().hex[:length]

    def isJavaScriptFunctionName(value) -> bool:
        return re.search("^[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*$", value)
    
    def isDashboardsProduct() -> bool:
        try:
            from stimulsoft_dashboards.report.StiDashboard import StiDashboard
        except Exception as e:
            return False
        
        return True
